/*
  Macro          Case
  Author         Carlo Hogeveen
  Website        eCarlo.nl/tse
  Compatability  TSE Pro v3 upwards, all TSE variants
  Version        v1   12 Jan 2023

  TSE's existing functions Upper(), Lower(), Flip() and existing macro "capital"
  work on a block if one is marked, and otherwise the functions work on the
  current character and do not advance the cursor.

  This tool provides handier versions of that functionality, in my opinion:
    "upper" and "lower":
      If the cursor is in a marked a block,
      then      they work the same as Upper and Lower(),
                making the whole block upper-case or lower-case,
      otherwise they change the case of the current word
                and advance the cursor to the next word.
    "flip":
      If the cursor is in a marked a block,
      then      it works the same as Flip(),
                flipping the case of the whole block,
      otherwise it does a Flip()
                and advances the cursor to the next character.
    "capitalize"
      If the cursor is in a marked a block,
      then      it capitalizes all words that are wholly within the block,
      otherwise it capitalizes the current word, if any,
                and advances the cursor to the next word.


  INSTALLATION

  Just copy this file to TSE's "mac" directory, and compile it there, for
  example by opening it there in TSE and applying the Macro Compile menu.


  EXAMPLES                  (Assume cursor on first example character.)

  <Ctrl 1>   ExecMacro('Case upper')        // HelloWorld -> HELLOWORLD
  <Ctrl 2>   ExecMacro('Case lower')        // HelloWorld -> helloworld
  <Ctrl 3>   ExecMacro('Case flip')         // HelloWorld -> helloWorld
  <Ctrl 4>   ExecMacro('Case capitalize')   // helloworld -> Helloworld


*/


// Global constants and semi-constants
string MACRO_NAME [MAXSTRINGLEN] = ''


string proc get_blocked_word_part()
  string word [MAXSTRINGLEN] = ''
  if  isCursorInBlock()
  and GetWord() <> ''
    word = GetText(CurrPos(), 1)
    PushLocation()
    while Left()
    and   isCursorInBlock()
    and   GetWord() <> ''
      word = GetText(CurrPos(), 1) + word
    endwhile
    PopLocation()
    PushLocation()
    while Right()
    and   isCursorInBlock()
    and   GetWord() <> ''
      word = word + GetText(CurrPos(), 1)
    endwhile
    PopLocation()
  endif
  return(word)
end get_blocked_word_part


proc capitalize_current_word()
  integer first_letter = TRUE
  if GetWord() == ''
    WordRight()
  else
    while Left()
    and   GetWord() <> ''
    endwhile
    if GetWord() == ''
      Right()
    endif
    while GetWord() <> ''
    and   CurrPos() <  MAXLINELEN
      if first_letter
        InsertText(Upper(GetText(CurrPos(), 1)), _OVERWRITE_)
        first_letter = FALSE
      else
        InsertText(Lower(GetText(CurrPos(), 1)), _OVERWRITE_)
      endif
    endwhile
    WordRight()
  endif
end capitalize_current_word


proc capitalize_block_or_word()
  if isCursorInBlock()
    PushLocation()
    GotoBlockBegin()
    if isCursorInBlock() == _LINE_
      BegLine()
    endif
    repeat
      if  isCursorInBlock()
      and GetWord() <> ''
      and Length(GetWord()) == Length(get_blocked_word_part())
        capitalize_current_word()
      else
        WordRight()
      endif
    until  CurrLine() >  Query(BlockEndLine)
       or (CurrLine() == Query(BlockEndLine) and CurrCol() > Query(BlockEndCol))
    PopLocation()
  else
    if GetWord() == ''
      WordRight()
    else
      capitalize_current_word()
    endif
  endif
end capitalize_block_or_word


proc do_action(string action)
  case action
    when 'upper'
      Upper()
    when 'lower'
      Lower()
    when 'flip'
      Flip()
  endcase
end do_action


proc case_block_or_word(string action)
  if isCursorInBlock()
    do_action(action)
  else
    PushBlock()
    UnMarkBlock()
    MarkWord()
    do_action(action)
    UnMarkBlock()
    PopBlock()
    WordRight()
  endif
end case_block_or_word


proc flip_block_or_character()
  Flip()
  if not isCursorInBlock()
    NextChar()
  endif
end flip_block_or_character


proc WhenPurged()
end WhenPurged


proc WhenLoaded()
  MACRO_NAME = SplitPath(CurrMacroFilename(), _NAME_)
end WhenLoaded


proc Main()
  string action [MAXSTRINGLEN] = Lower(Trim(Query(MacroCmdLine)))

  case action
    when 'upper', 'lower'
      case_block_or_word(action)
    when 'flip'
      flip_block_or_character()
    when 'capitalize'
      capitalize_block_or_word()
    otherwise
      Warn(MACRO_NAME; 'error: Unknown parameter "', Trim(Query(MacroCmdLine)),
           '".')
  endcase
end Main

